#!/bin/bash

#
# Priorities in CFQ are weighted by 1/5th in the following manner:
#
# 0 - 180
# 1 - 160
# 2 - 140
# 3 - 120
# 4 - 100
# 5 - 80
# 6 - 60
# 7 - 40
#
prio_weight=([0]=180 160 140 120 100 80 60 40)

results_dir=output
if [ $# -eq 1 ]; then
	results_dir=$1
fi

function calc_ideal_xferred()
{
	total_xferred=$1
	total_prio=$2
	this_prio=$3

	echo "((($total_xferred*100)*${prio_weight[$this_prio]})/$total_prio)/100" | bc
}

function calc_diff()
{
	real=$1
	ideal=$2

	((real=$real*100))
	((diff=$real/$ideal))
	((diff-=100))

	echo $diff
}

for dir in `ls $results_dir`; do
	job=`basename $dir`
	pushd $results_dir/$job

	unset be_priorities
	unset be_xferred
	unset rt_priorities
	unset rt_xferred

	declare -a be_priorities
	declare -a be_xferred
	declare -a rt_priorities
	declare -a rt_xferred

	names=`cat fio.out | grep test | awk -F\; '{print $1}'`
	total_prio=0
	for name in $names; do
		# The naming convention used for the tests is:
		# <name>.(be|rt)<prio>[*]
		# For example:  test7.be0
		# or            test3.rt0
		# or even       test3.be4.1
		class_n_prio=`echo $name | awk -F. '{print $2}'`
		class=`echo $class_n_prio | sed -e 's/\([a-z]\+\)[0-9]\+/\1/g'`
		prio=`echo $class_n_prio | sed -e 's/[a-z]\+//g'`

		# append theh new prio value to the end of the
		# appropriate array
		if [ $class == "be" ]; then
			index=${#be_priorities[@]}
			be_priorities[$index]=$prio
		else
			index=${#rt_priorities[@]}
			rt_priorities[$index]=$prio
		fi

		((total_prio+=${prio_weight[$prio]}))
	done
	echo "total priority: $total_prio"

	kb_read=`cat fio.out | grep test | awk -F\; '{print $4}'`
	total_xferred=0
	for datum in $kb_read; do
		if [ $class == "be" ]; then
			index=${#be_xferred[@]}
			be_xferred[$index]=$datum
		else
			index=${#rt_xferred[@]}
			rt_xferred[$index]=$datum
		fi
		((total_xferred+=$datum))
	done
	echo "total data transferred: $total_xferred"

	echo -e "class\tprio\tideal\txferred\t%diff"
	len=${#rt_priorities[@]}
	if [ $len -ne 0 ]; then
		((len--))
		for i in `seq 0 $len`; do
			ideal=`calc_ideal_xferred $total_xferred $total_prio ${rt_priorities[$i]}`
			diff=`calc_diff ${rt_xferred[$i]} $ideal`
			echo -ne "rt\t${rt_priorities[$i]}\t$ideal\t"
			echo -e "${rt_xferred[$i]}\t$diff"
#			echo -e "rt\t${rt_priorities[$i]}\t$ideal\t${rt_xferred[$i]}\t"
		done
	fi

	len=${#be_priorities[@]}
	if [ $len -ne -0 ]; then
		((len--))
		for i in `seq 0 $len`; do
			ideal=`calc_ideal_xferred $total_xferred $total_prio ${be_priorities[$i]}`
			diff=`calc_diff ${be_xferred[$i]} $ideal`
			echo -e "be\t${be_priorities[$i]}\t$ideal\t${be_xferred[$i]}\t$diff"
		done
	fi
	popd
done
