#!/bin/bash

DEV=
MNT=/mnt/test
do_blktrace=0
results_dir=output

function usage()
{
	echo "Usage: $0 <-d dev> [-m mount-point] [-r results-dir] [-b]"
	echo -e "\tb\tWhen specified, enables blktrace capture;  default state is disabled."
	exit 1
}

while getopts "d:m:r:b" opt "$@"
do
	case $opt in
	"d")
		DEV=$OPTARG
		;;
	"m")
		MNT=$OPTARG
		;;
	"b")
		do_blktrace=1
		;;
	"r")
		results_dir=$OPTARG
		;;
	"?")
		usage
	esac
done

if [ -z "$DEV" ]; then
	usage
fi

echo -n "Using device $DEV, mount-point $MNT, blktracing is "
if [ $do_blktrace -eq 1 ]; then
	echo "enabled"
else
	echo "disabled"
fi

function mount_debugfs()
{
	# Make sure debugfs is loaded
	grep -q debugfs /proc/mounts
	if [ $? -eq 1 ]; then
		mount -t debugfs none /sys/kernel/debug
		if [ $? -ne 0 ]; then
			echo "Failed to mount debugfs!"
			do_blktrace=0
		fi
	fi
}

function start_blktrace()
{
	dev=$1
	devname=`basename $dev`
	if [ $do_blktrace -eq 1 ]; then
		blktrace -d $1 -o $2/$devname >&/dev/null &
	fi
}

function stop_blktrace()
{
	if [ $do_blktrace -eq 1 ]; then
		# There is currently a bug in blktrace whereby stopping the
		# trace and then tearing it down does not unblock the running
		# blktrace.  So, it just hangs out forever.  Until I fix this,
		# it's just as easy to simply kill off the blktrace process.
#		blktrace -d $1 -k
#		if [ $? -ne 0 ]; then
			killall blktrace
#		fi
	fi
}

if [ $do_blktrace -ne 0 ]; then
	mount_debugfs
fi

if [ ! -d $MNT ]; then
	mkdir -p $MNT
fi

for job in `ls *.fio`; do
	umount /mnt/test >& /dev/null
	mount $DEV $MNT
	if [ $? -ne 0 ]; then
		echo "Failed to mount $DEV on $MNT"
		exit 1
	fi

	fio_cmd=`fio --showcmd $job`
	fio_cmd=`echo $fio_cmd | sed -e 's/^fio //g'`
	fio_cmd="fio --minimal --directory=$MNT $fio_cmd"

	echo "Running job file $job"
	mkdir -p $results_dir/$job
	start_blktrace $DEV $results_dir/$job
	$fio_cmd > $results_dir/$job/fio.out
	stop_blktrace $DEV
done

umount /mnt/test >& /dev/null

echo "Done"
